import java.io.*;
import javax.swing.*;
import java.util.*;

public class analyser
{
	private ArrayList<song> musicLibrary = new ArrayList<song>();
	private ArrayList<song> videoLibrary = new ArrayList<song>();
	private ArrayList<String> rawData = new ArrayList<String>();
	private ArrayList<String> printList = new ArrayList<String>();
	private song song;
	private int mostPlayed = 0;
	
	public analyser()
	{
		Calendar rightNow = Calendar.getInstance();		
		System.out.println("\n");
		readFile();
		scanFile();
		analyseMusic();
		analyseVideo();
	}
	
	public void readFile() {
		String store;
		try
	    {
	        FileReader reader = null;
			try
	        {   
				String fileLocation = System.getProperty("user.home") + "/Music/iTunes/iTunes Music Library.xml";
				reader = new FileReader(fileLocation);
				Scanner in = new Scanner(reader);
				int linesRead = 1;
				while(in.hasNextLine()) {
					store = in.nextLine();
					if (store.contains("<array>")) {
						break;
					}
					linesRead++;
					rawData.add(store);
				}
	        }
	        finally {
	            if (reader != null) {
	                reader.close();
	            }
	        }
	    }
	    catch (FileNotFoundException exception) {
	        System.out.println("File not found: " + exception);
	    }
	    catch (IOException exception) {
	        System.out.println("Error processing file: " + exception);
	    }
	}

	public void scanFile() {		
		int musicCount = 0;
		int videoCount = 0;
		int length = 0;
		int trackID = 0;
		int noCount = 0;
		int playCount = 0;
		int ignoreDict = 0;
		boolean titleFound = false;
		boolean playCountFound = false;
		String name = "";
		String kind = "";
		for (String s: rawData) {
			if (s.contains("<dict>") && ignoreDict < 2) {
				ignoreDict++;
			}
			if (s.contains("<dict>") && ignoreDict == 2) {
				titleFound = true;
			}
			if (s.contains("<key>Name</key><string>") && titleFound == true) {
				name = removeTail(s.substring(26));
				musicCount++;
			}
			
			//video detection
			if (s.contains("<key>Kind</key><string>") && titleFound == true) {
				kind = removeTail(s.substring(26));
				musicCount++;
			}
			
			if (s.contains("<key>Track ID</key><integer>") && titleFound == true) {
				trackID = Integer.parseInt(removeTail(s.substring(31)));
			}
			if (s.contains("<key>Total Time</key><integer>") && titleFound == true) {
				length = Integer.parseInt(removeTail(s.substring(33)));
			}
			if (s.contains("<key>Play Count</key><integer>") && titleFound == true) {
				playCountFound = true;
				playCount = Integer.parseInt(removeTail(s.substring(33)));
			}
			if (s.contains("</dict>") && titleFound == true)
			{
				if (playCountFound == false) {
					noCount++;
					playCount = 0;
				}
				
				if (playCount > mostPlayed) {
					mostPlayed = playCount;
				}
				
				if (!kind.contains("MPEG-4 video file")) {	
					musicLibrary.add(new song(trackID, name, length, playCount));
				}
				else {
					videoCount++;
					videoLibrary.add(new song(trackID, name, length, playCount));
				}
				
				titleFound = false;
				playCountFound = false;
			}
		}
	}
	
	public void analyseMusic() {
		stacker[] countStack = new stacker[mostPlayed+1];
		stacker st;
		for (int j = 0; j < mostPlayed+1; j++) {
			st = new stacker();
			countStack[j] = st;
		}
		
		for (song s: musicLibrary) {
			st = countStack[s.getPlayCount()];
			st.getStack().add(s);
		}
		
		int totalSongs = 0;
		int songs = 0;
		int totalCountNo = 0;
		int countNo = 0;
		long totalTime = 0;
		long time = 0;
		long totalTimeCount = 0;
		long timeCount = 0;
		
		for (int i = 0; i < countStack.length; i++) {
			songs = countStack[i].getStack().size();
			countNo = countStack[i].getStack().size()*i;
			time = countStack[i].totalLength();
			timeCount = countStack[i].totalLength()*i;
			
			totalSongs += songs;
			totalCountNo += countNo;
			totalTime += time;
			totalTimeCount += timeCount;
		}
		
		System.out.println("___Music Analysis Results__________________________________________________________\n");

		System.out.println("            Total Songs: "+totalSongs);
		System.out.println("      Total Play Counts: "+totalCountNo);
		System.out.println("Total Library Play Time: "+millisToTime(totalTime, 2)+" ("+millisToTime(totalTime, 1)+").");
		System.out.println("   Total Listening Time: "+millisToTime(totalTimeCount, 2)+" ("+millisToTime(totalTimeCount, 1)+").");
		
		System.out.println("___________________________________________________________________________________\n");
	}
	
	public void analyseVideo() {
		stacker[] countStack = new stacker[mostPlayed+1];
		stacker st;
		for (int j = 0; j < mostPlayed+1; j++)
		{
			st = new stacker();
			countStack[j] = st;
		}
		
		for (song s: videoLibrary)
		{
			st = countStack[s.getPlayCount()];
			st.getStack().add(s);
		}
		
		int totalSongs = 0;
		int songs = 0;
		int totalCountNo = 0;
		int countNo = 0;
		long totalTime = 0;
		long time = 0;
		long totalTimeCount = 0;
		long timeCount = 0;
		
		for (int i = 0; i < countStack.length; i++)	{
			songs = countStack[i].getStack().size();
			countNo = countStack[i].getStack().size()*i;
			time = countStack[i].totalLength();
			timeCount = countStack[i].totalLength()*i;
			
			totalSongs += songs;
			totalCountNo += countNo;
			totalTime += time;
			totalTimeCount += timeCount;				
		}

		System.out.println("___Video Analysis Results__________________________________________________________\n");

		System.out.println("           Total Videos: "+totalSongs);
		System.out.println("      Total Play Counts: "+totalCountNo);
		System.out.println("Total Library Play Time: "+millisToTime(totalTime, 2)+" ("+millisToTime(totalTime, 1)+").");
		System.out.println("    Total Watching Time: "+millisToTime(totalTimeCount, 2)+" ("+millisToTime(totalTimeCount, 1)+").");
		
		System.out.println("___________________________________________________________________________________\n");
	}

	
	public String removeTail(String input)
	{
		String result = input;

		for (int i = 0; i < result.length(); i++)
		{			
			if (result.substring(i, i+1).equalsIgnoreCase("<"))
			{
				result = input.substring(0, i);
				return result;
			}
		}
		return null;
	}
	
	
	public String millisToTime(long time, int choice)
	{
		long milli = 0;
		long sec = 0;
		long min = 0;
		long hour = 0;
		long day = 0;
		
		long remainder = 0;
		
		milli = time%1000;
		remainder = (time-milli)/1000;
		
		sec = remainder%60;
		remainder -= sec;
		remainder /= 60;
		
		min = remainder%60;
		remainder -= min;
		remainder /= 60;
		
		hour = remainder%24;
		remainder -= hour;
		day = remainder/24;
		if (choice == 1)
			return day+":"+hour+":"+min+":"+sec+"."+milli;
		else if (choice == 2)
			return day+" days, "+hour+" hours, "+min+" minutes "+sec+" seconds";
		else
			return null;
	}
}